<?php

namespace Drupal\required_api;

use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Form\FormErrorHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Provides an entity form error handler that clears unnecessary form errors.
 */
class RequiredApiFormErrorHandler implements FormErrorHandlerInterface {

  /**
   * Constructs a RequiredApiFormErrorHandler object.
   *
   * @param \Drupal\Core\Form\FormErrorHandlerInterface $formErrorHandler
   *   The inner form error handler service.
   * @param \Drupal\required_api\RequiredManager $requiredManager
   *   The required manager.
   */
  public function __construct(
    protected FormErrorHandlerInterface $formErrorHandler,
    protected RequiredManager $requiredManager,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public function handleFormErrors(array &$form, FormStateInterface $form_state) {
    $entityForm = $form_state->getFormObject();
    if (!$entityForm instanceof ContentEntityFormInterface) {
      $this->formErrorHandler->handleFormErrors($form, $form_state);
      return $this;
    }

    // Get an entity with up-to-date values.
    // We clone the form state to make sure any changes are not persisted.
    $entity = $entityForm->buildEntity($form, clone $form_state);
    assert($entity instanceof ContentEntityInterface);

    // Get the errors and clear them.
    $errors = $form_state->getErrors();
    $form_state->clearErrors();

    foreach ($entity->getFieldDefinitions() as $fieldName => $fieldDefinition) {
      if ($fieldDefinition instanceof BaseFieldDefinition) {
        continue;
      }

      // Determine if the field is required.
      $plugin = $this->requiredManager->getInstance(['field_definition' => $fieldDefinition]);
      $required = $plugin->isRequired($fieldDefinition, $entity);
      if ($required === TRUE) {
        continue;
      }

      // Remove all required error messages for this field.
      $required_error_messages = [
        '@name field is required.',
        'The %field date is required.',
      ];

      foreach ($errors as $name => $message) {
        if ($message instanceof TranslatableMarkup) {
          $rawMessage = $message->getUntranslatedString();
        }
        else {
          $rawMessage = $message;
        }

        if (!in_array($rawMessage, $required_error_messages, TRUE)) {
          continue;
        }

        if ($name !== $fieldName && substr($name, 0, strpos($name, ']')) !== $fieldName) {
          continue;
        }

        unset($errors[$name]);
      }
    }

    // Re-add the remaining errors.
    foreach ($errors as $name => $message) {
      $form_state->setErrorByName($name, $message);
    }

    $this->formErrorHandler->handleFormErrors($form, $form_state);
    return $this;
  }

}
